@@ -35,8 +35,8 @@
 
 #include <wx/filename.h>
 
-#include "mygdal/ogr_s57.h"
-#include "mygdal/cpl_csv.h"
+#include "gdal/cpl_csv.h"
+#include "ogr_s57.h"
 #include "chartbase.h"
 
 #include <string.h>
