@@ -27,25 +27,33 @@
 #define _PLUGIN_H_
 
 #ifndef DECL_EXP
-#ifdef __WXMSW__
+#if defined(__WXMSW__) || defined(__CYGWIN__)
 #  define DECL_EXP     __declspec(dllexport)
+#elif defined __GNUC__ && __GNUC__ >= 4
+#  define DECL_EXP     __attribute__((visibility("default")))
+#elif defined __WXOSX__
+#  define DECL_EXP     __attribute__((visibility("default")))
 #else
-# ifdef __GNUC__
-# define DECL_EXP       __attribute__((visibility("default")))
-# endif
+#  define DECL_EXP
 #endif
 #endif
 
-#ifdef __WXMSW__
-#ifdef MAKING_PLUGIN
+#if defined(__WXMSW__) && defined(MAKING_PLUGIN)
 #  define DECL_IMP     __declspec(dllimport)
-#endif
+#else 
+#  define DECL_IMP
 #endif    
 
 #include <wx/xml/xml.h>
+#include <wx/dcmemory.h>
+#include <wx/dialog.h>
+#include <wx/event.h>
+#include <wx/menuitem.h>
+#include <wx/gdicmn.h>
+
 
 #ifdef ocpnUSE_SVG
-#include "wxSVG/svg.h"
+#include <wx/bitmap.h>
 #endif // ocpnUSE_SVG
 
 #include <memory>
@@ -58,7 +66,7 @@ class wxGLContext;
 //    PlugIns conforming to API Version less then the most modern will also
 //    be correctly supported.
 #define API_VERSION_MAJOR           1
-#define API_VERSION_MINOR           16
+#define API_VERSION_MINOR           17
 
 //    Fwd Definitions
 class       wxFileConfig;
@@ -156,6 +164,16 @@ class PlugIn_Position_Fix_Ex
             int    nSats;
 };
 
+class Plugin_Active_Leg_Info
+{
+public:
+  double Xte;               // Left side of the track -> negative XTE
+  double Btw;
+  double Dtw;
+  wxString wp_name;         // Name of destination waypoint for active leg
+  bool arrival;             // True when within arrival circle
+};
+
 //    Describe AIS Alarm state
 enum plugin_ais_alarm_type
 {
@@ -536,7 +554,6 @@ class DECL_EXP opencpn_plugin_115 : public opencpn_plugin_114
 public:
     opencpn_plugin_115(void *pmgr);
     virtual ~opencpn_plugin_115();
-
 };
 
 class DECL_EXP opencpn_plugin_116 : public opencpn_plugin_115
@@ -550,6 +567,28 @@ public:
 
 };
 
+class DECL_EXP opencpn_plugin_117 : public opencpn_plugin_116
+{
+public:
+    opencpn_plugin_117(void *pmgr);
+    /*
+     * Forms a semantic version together with GetPlugInVersionMajor() and
+     * GetPlugInVersionMinor(), see https://semver.org/
+     */
+    virtual int GetPlugInVersionPatch();
+
+    /** Post-release version part, extends the semver spec. */
+    virtual int GetPlugInVersionPost();
+
+    /** Pre-release tag version part, see GetPlugInVersionPatch() */
+    virtual const char* GetPlugInVersionPre();
+
+    /** Build version part  see GetPlugInVersionPatch(). */
+    virtual const char* GetPlugInVersionBuild();
+
+    /*Provide active leg data to plugins*/
+    virtual void SetActiveLegInfo(Plugin_Active_Leg_Info &leg_info);
+};
 //------------------------------------------------------------------
 //      Route and Waypoint PlugIn support
 //
@@ -1097,6 +1136,7 @@ extern DECL_EXP void SetCanvasTilt(double tilt);
 extern DECL_EXP bool PlugInPlaySoundEx( wxString &sound_file, int deviceIndex=-1 );
 extern DECL_EXP void AddChartDirectory( wxString &path );
 extern DECL_EXP void ForceChartDBUpdate();
+extern DECL_EXP void ForceChartDBRebuild();
 
 extern  DECL_EXP wxString GetWritableDocumentsDir( void );
 extern  DECL_EXP wxDialog *GetActiveOptionsDialog();
@@ -1172,7 +1212,7 @@ enum OCPN_DLDialogStyle
 
 /*   Synchronous (Blocking) download of a single file  */
 
-extern DECL_EXP _OCPN_DLStatus OCPN_downloadFile( const wxString& url, const wxString &outputFile, 
+extern DECL_EXP _OCPN_DLStatus OCPN_downloadFile( const wxString& url, const wxString &outputFile,
                                        const wxString &title, const wxString &message, 
                                        const wxBitmap& bitmap,
                                        wxWindow *parent, long style, int timeout_secs);
@@ -1250,6 +1290,13 @@ extern   DECL_IMP wxEventType wxEVT_DOWNLOAD_EVENT;
 extern   DECL_EXP wxEventType wxEVT_DOWNLOAD_EVENT;
 #endif
 
+
+/* API 1.14  */
+/* API 1.14  adds some more common functions to avoid unnecessary code duplication */
+
+bool LaunchDefaultBrowser_Plugin( wxString url );
+    
+    
 // API 1.14 Extra canvas Support
 
 /* Allow drawing of objects onto other OpenGL canvases */
@@ -1260,7 +1307,7 @@ extern DECL_EXP bool PlugInSetFontColor(const wxString TextElement, const wxColo
 extern DECL_EXP double PlugInGetDisplaySizeMM();
 
 // 
-extern DECL_EXP wxFont* FindOrCreateFont_PlugIn( int point_size, wxFontFamily family, 
+extern DECL_EXP wxFont* FindOrCreateFont_PlugIn( int point_size, wxFontFamily family,
                     wxFontStyle style, wxFontWeight weight, bool underline = false,
                     const wxString &facename = wxEmptyString,
                     wxFontEncoding encoding = wxFONTENCODING_DEFAULT );
@@ -1319,6 +1366,10 @@ extern DECL_EXP int GetCanvasCount( );
 extern DECL_EXP bool CheckMUIEdgePan_PlugIn( int x, int y, bool dragging, int margin, int delta, int canvasIndex );
 extern DECL_EXP void SetMUICursor_PlugIn( wxCursor *pCursor, int canvasIndex );
 
+// API 1.17
+//
+extern DECL_EXP wxRect GetMasterToolbarRect();
+
 enum SDDMFORMAT
 {
     DEGREES_DECIMAL_MINUTES = 0,
@@ -1329,4 +1380,7 @@ enum SDDMFORMAT
 
 extern DECL_EXP int GetLatLonFormat(void);
 
+// API 1.17
+extern "C"  DECL_EXP void ZeroXTE();
+
 #endif //_PLUGIN_H_
