@@ -107,6 +107,19 @@ static bool RecordIsWind(GribRecord *rec)
          rec->getDataType()==GRB_WIND_DIR || rec->getDataType()==GRB_WIND_SPEED;
 }
 
+//---------------------------------------------------------------------------------
+static bool RecordIsGust(GribRecord *rec)
+{
+  return rec->getDataType()==GRB_WIND_GUST_VX || rec->getDataType()==GRB_WIND_GUST_VY ||
+		 rec->getDataType()==GRB_WIND_GUST;
+}
+
+static bool RecordIsCurrent(GribRecord *rec)
+{
+  return rec->getDataType()==GRB_UOGRD || rec->getDataType()==GRB_VOGRD ||
+         rec->getDataType()==GRB_CUR_DIR || rec->getDataType()==GRB_CUR_SPEED;
+}
+
 void GribReader::readAllGribRecords()
 {
     //--------------------------------------------------------
@@ -176,17 +189,18 @@ void GribReader::readAllGribRecords()
         if (firstdate== -1)
 	    firstdate = rec->getRecordCurrentDate();
 
-        if ((rec->getDataType()==GRB_PRESSURE && rec->getLevelType()==LV_MSL && rec->getLevelValue()==0)
-                    || ( RecordIsWind(rec) && rec->getLevelType()==LV_ABOV_GND && rec->getLevelValue()==10)
-                    || ( RecordIsWind(rec) && rec->getLevelType()==LV_ISOBARIC //wind at x hpa
-                    && (  rec->getLevelValue()==850
-			|| rec->getLevelValue()==700
-			|| rec->getLevelValue()==500
-			|| rec->getLevelValue()==300 ) ) )
+        if ((rec->getDataType()==GRB_PRESSURE && rec->getLevelValue()==0 &&
+                (rec->getLevelType()==LV_MSL || rec->getLevelType()==LV_GND_SURF)
+            )
+            || ( RecordIsWind(rec) && rec->getLevelType()==LV_ABOV_GND && rec->getLevelValue()==10)
+                || ( RecordIsWind(rec) && rec->getLevelType()==LV_ISOBARIC //wind at x hpa
+                    && (  rec->getLevelValue()==850 || rec->getLevelValue()==700
+			|| rec->getLevelValue()==500 || rec->getLevelValue()==300 
+                       ) 
+               ) )
             storeRecordInMap(rec);
 
-        else if( (rec->getDataType()==GRB_WIND_GUST
-                    && rec->getLevelType()==LV_GND_SURF && rec->getLevelValue()==0) )
+        else if( (RecordIsGust(rec) && rec->getLevelType()==LV_GND_SURF && rec->getLevelValue()==0) )
             storeRecordInMap(rec);
 
         else if( RecordIsWind(rec) && rec->getLevelType()==LV_GND_SURF)
@@ -212,29 +226,35 @@ void GribReader::readAllGribRecords()
 			&& rec->getLevelType()==LV_GND_SURF && rec->getLevelValue()==0)
             storeRecordInMap(rec);
 
-        else if(rec->getDataType()==GRB_CLOUD_TOT                //cloud cover
+        else if((rec->getDataType()==GRB_CLOUD_TOT                //cloud cover
+                 || rec->getDataType()==GRB_COMP_REFL)
                     && rec->getLevelType()==LV_ATMOS_ALL && rec->getLevelValue()==0 )                          
             storeRecordInMap(rec);
-
         else if( rec->getDataType() == GRB_HTSGW )               // Significant Wave Height
             storeRecordInMap(rec);
 
-        else if( rec->getDataType() == GRB_WVPER )               // Waves period
+        else if( rec->getDataType() == GRB_PER )                 // Combined Wind Waves and Swell period
             storeRecordInMap(rec);
 
-        else if( rec->getDataType() == GRB_WVDIR )               // Wind Wave Direction
+        else if( rec->getDataType() == GRB_DIR )                 // Combined Wind Waves and Swell Direction
             storeRecordInMap(rec);
 
-        else if( rec->getDataType() == GRB_WVHGT )               // Wave Height
+        else if( rec->getDataType() == GRB_WVHGT )               // Wind Wave Height
             storeRecordInMap(rec);
                 
+        else if( rec->getDataType() == GRB_WVPER )               // Wind Waves period
+            storeRecordInMap(rec);
+
+        else if( rec->getDataType() == GRB_WVDIR )               // Wind Waves Direction
+            storeRecordInMap(rec);
+
         else if( rec->getDataType() == GRB_CRAIN )               // Catagorical Rain  1/0
             storeRecordInMap(rec);
 
         else if ((rec->getDataType()==GRB_WTMP) && (rec->getLevelType()==LV_GND_SURF) && (rec->getLevelValue()==0))
             storeRecordInMap(rec);                             // rtofs Water Temp + translated gfs Water Temp  
 
-        else if( (rec->getDataType()==GRB_UOGRD || rec->getDataType()==GRB_VOGRD))          // rtofs model sea current current
+        else if( RecordIsCurrent(rec))          // rtofs model sea current current
             storeRecordInMap(rec);
 
         else if(rec->getDataType() == GRB_CAPE && rec->getLevelType()==LV_GND_SURF && rec->getLevelValue()==0) //Potential energy
@@ -414,6 +434,8 @@ void  GribReader::copyMissingWaveRecords ()
 	copyMissingWaveRecords (GRB_HTSGW, LV_GND_SURF, 0);
 	copyMissingWaveRecords (GRB_WVDIR, LV_GND_SURF,0);
     copyMissingWaveRecords (GRB_WVPER, LV_GND_SURF,0);
+	copyMissingWaveRecords (GRB_DIR, LV_GND_SURF,0);
+    copyMissingWaveRecords (GRB_PER, LV_GND_SURF,0);
 }
 
 //---------------------------------------------------------------------------------
@@ -424,46 +446,60 @@ void GribReader::readGribFileContent()
 
     createListDates();
 //    hoursBetweenRecords = computeHoursBeetweenGribRecords();
+	// XXX should it be done after reading all files, rather than per file?
+	if ( getNumberOfGribRecords(GRB_WIND_GUST, LV_GND_SURF, 0) == 0) {
+		for (auto date :setAllDates ) {
+			GribRecord *recX = getGribRecord(GRB_WIND_GUST_VX, LV_GND_SURF, 0, date);
+			if (recX == nullptr)
+				continue;
 
-
+			GribRecord *recY = getGribRecord(GRB_WIND_GUST_VY, LV_GND_SURF, 0, date);
+			if (recY == nullptr)
+			    continue;
+			GribRecord *rec = GribRecord::MagnitudeRecord(*recX, *recY);
+			rec->setDataType(GRB_WIND_GUST);
+			storeRecordInMap(rec);
+		}
+    }
 	//-----------------------------------------------------
 	// Are dewpoint data in file ?
 	// If no, compute it with Magnus-Tetens formula, if possible.
 	//-----------------------------------------------------
 	dewpointDataStatus = DATA_IN_FILE;
-	if (getNumberOfGribRecords(GRB_DEWPOINT, LV_ABOV_GND, 2) == 0)
+	if ( getNumberOfGribRecords(GRB_DEWPOINT, LV_ABOV_GND, 2) != 0)
+	    return;
+
+    dewpointDataStatus = NO_DATA_IN_FILE;
+    if ( getNumberOfGribRecords(GRB_HUMID_REL, LV_ABOV_GND, 2) == 0
+		   || getNumberOfGribRecords(GRB_TEMP, LV_ABOV_GND, 2) == 0)
+		return;
+
+	dewpointDataStatus = COMPUTED_DATA;
+	for (auto iter :setAllDates )
 	{
-		dewpointDataStatus = NO_DATA_IN_FILE;
-		if (  getNumberOfGribRecords(GRB_HUMID_REL, LV_ABOV_GND, 2) > 0
-		   && getNumberOfGribRecords(GRB_TEMP, LV_ABOV_GND, 2) > 0)
+		time_t date = iter;
+		GribRecord *recModel = getGribRecord(GRB_TEMP,LV_ABOV_GND,2,date);
+		if (recModel == nullptr)
+		    continue;
+
+        // Crée un GribRecord avec les dewpoints calculés
+		GribRecord *recDewpoint = new GribRecord(*recModel);
+        recDewpoint->setDataType(GRB_DEWPOINT);
+		for (zuint i=0; i<(zuint)recModel->getNi(); i++)
 		{
-			dewpointDataStatus = COMPUTED_DATA;
-			std::set<time_t>::iterator iter;
-			for (iter=setAllDates.begin(); iter!=setAllDates.end(); iter++)
+		    for (zuint j=0; j<(zuint)recModel->getNj(); j++)
 			{
-				time_t date = *iter;
-				GribRecord *recModel = getGribRecord(GRB_TEMP,LV_ABOV_GND,2,date);
-				if (recModel != NULL)
-				{
-					// Crée un GribRecord avec les dewpoints calculés
-					GribRecord *recDewpoint = new GribRecord(*recModel);
-                                        recDewpoint->setDataType(GRB_DEWPOINT);
-					for (zuint i=0; i<(zuint)recModel->getNi(); i++)
-					    for (zuint j=0; j<(zuint)recModel->getNj(); j++)
-					    {
-					        double x, y;
-					        recModel->getXY(i, j, &x, &y);
-						double dp = computeDewPoint(x, y, date);
-						recDewpoint->setValue(i, j, dp);
-                                            }
-                                        storeRecordInMap(recDewpoint);
-				}
-			}
+			    double x, y;
+			    recModel->getXY(i, j, &x, &y);
+				double dp = computeDewPoint(x, y, date);
+				recDewpoint->setValue(i, j, dp);
+            }
 		}
+        storeRecordInMap(recDewpoint);
 	}
-	//-----------------------------------------------------
 }
 
+
 //---------------------------------------------------
 int GribReader::getDewpointDataStatus(int /*levelType*/,int /*levelValue*/)
 {
