@@ -27,6 +27,10 @@
 #include <wx/tokenzr.h>
 #include <wx/clipbrd.h>
 
+#ifdef __OCPN__ANDROID__
+#include "androidUTIL.h"
+#endif
+
 #include "AISTargetListDialog.h"
 #include "ais.h"
 #include "AIS_Decoder.h"
@@ -56,9 +60,7 @@ extern MyFrame *gFrame;
 extern wxString g_default_wp_icon;
 extern Select *pSelect;
 extern RouteManagerDialog *pRouteManagerDialog;
-extern bool g_bAISShowTracks;
 extern bool g_btouch;
-extern bool g_bresponsive;
 
 IMPLEMENT_CLASS ( AISTargetListDialog, wxPanel )
 
@@ -313,8 +315,12 @@ AISTargetListDialog::AISTargetListDialog( wxWindow *parent, wxAuiManager *auimgr
 
     if( m_pAuiManager ) {
         wxAuiPaneInfo paneproto =
-                wxAuiPaneInfo().Name( _T("AISTargetList") ).CaptionVisible( true ).Float().FloatingPosition( 50, 50 )
-                .FloatingSize(400, 200).BestSize(700, GetCharHeight() * 10);
+                wxAuiPaneInfo().Name( _T("AISTargetList") )
+                .CaptionVisible( true )
+                .Float()
+                .FloatingPosition( 50, 50 )
+                .FloatingSize(400, 200)
+                .BestSize(700, GetCharHeight() * 10);
                 
  
         //      Force and/or override any perspective information that is not applicable
@@ -329,13 +335,14 @@ AISTargetListDialog::AISTargetListDialog( wxWindow *parent, wxAuiManager *auimgr
         wxAuiPaneInfo &pane = m_pAuiManager->GetPane(_T("AISTargetList"));
         
         if(g_AisTargetList_perspective.IsEmpty()){
-            RecalculateSize();
+            if(!g_btouch)
+                RecalculateSize();
         }
         else{
             m_pAuiManager->LoadPaneInfo( g_AisTargetList_perspective, pane );
             m_pAuiManager->Update();
         }
-
+        
         pane = m_pAuiManager->GetPane(_T("AISTargetList"));     // Refresh the reference
         
         //  Some special setup for touch screens
@@ -343,9 +350,9 @@ AISTargetListDialog::AISTargetListDialog( wxWindow *parent, wxAuiManager *auimgr
             pane.Float();
             pane.Dockable( false );
             
-            wxSize screen_size = ::wxGetDisplaySize();
-            pane.FloatingSize(screen_size.x * 6/10, screen_size.y * 8/10);
-            pane.FloatingPosition(screen_size.x * 2/10, screen_size.y * 1/10);
+            wxSize screen_size = gFrame->GetClientSize();
+            pane.FloatingSize(screen_size.x * 8/10, screen_size.y * 8/10);
+            pane.FloatingPosition(screen_size.x * 1/10, screen_size.y * 1/10);
             m_pAuiManager->Update();
         }
         
@@ -745,7 +752,6 @@ void AISTargetListDialog::SetColorScheme()
 
 void AISTargetListDialog::OnPaneClose( wxAuiManagerEvent& event )
 {
-    wxAuiPaneInfo *pane = event.pane;
     if( event.pane->name == _T("AISTargetList") ) {
         g_AisTargetList_perspective = m_pAuiManager->SavePaneInfo( *event.pane );
     }
@@ -766,8 +772,12 @@ void AISTargetListDialog::Shutdown( void )
         Disconnect_decoder();
         pane.Show(false);
         m_pAuiManager->Update();
+#ifdef __OCPN__ANDROID__
+        GetParent()->Refresh( true );
+#endif        
         Destroy();
     }
+    
 }
 
 
@@ -1150,7 +1160,7 @@ void AISTargetListDialog::UpdateNVAISTargetList( void )
             m_pListCtrlAISTargets->DeleteAllItems();
 
         wxString count;
-        count.Printf( _T("%d"), m_pMMSI_array->GetCount() );
+        count.Printf( _T("%lu"), (unsigned long)m_pMMSI_array->GetCount() );
         m_pTextTargetCount->ChangeValue( count );
 
 #ifdef __WXMSW__
