@@ -49,7 +49,6 @@ extern wxString g_default_wp_icon;
 extern Select *pSelect;
 extern MyConfig *pConfig;
 extern RouteManagerDialog *pRouteManagerDialog;
-extern RouteList *pRouteList;
 extern TrackList *pTrackList;
 extern OCPNPlatform  *g_Platform;
 extern MyFrame *gFrame;
@@ -66,6 +65,7 @@ BEGIN_EVENT_TABLE ( AISTargetQueryDialog, wxFrame )
     EVT_CLOSE(AISTargetQueryDialog::OnClose)
     EVT_MOVE( AISTargetQueryDialog::OnMove )
     EVT_SIZE( AISTargetQueryDialog::OnSize )
+    EVT_CHAR_HOOK(AISTargetQueryDialog::OnKey)
 END_EVENT_TABLE()
 
 AISTargetQueryDialog::AISTargetQueryDialog()
@@ -108,12 +108,27 @@ void AISTargetQueryDialog::OnIdOKClick( wxCommandEvent& event )
     Close();
 }
 
+void AISTargetQueryDialog::OnKey( wxKeyEvent& ke )
+{
+    if ( ke.GetKeyCode() == WXK_ESCAPE )
+        Close( true );
+    else
+        ke.Skip(); 
+}
+
+
 void AISTargetQueryDialog::OnIdWptCreateClick( wxCommandEvent& event )
 {
     if( m_MMSI != 0 ) { //  Faulty MMSI could be reported as 0
         AIS_Target_Data *td = g_pAIS->Get_Target_Data_From_MMSI( m_MMSI );
         if( td ) {
-            wxString n =  wxString::Format(wxT("\"%s\"  %i "),td->ShipName,  td->MMSI).append(wxDateTime::Now().Format(wxT("%H:%M")));
+            wxString n0 = wxString::Format(wxT("%s"),td->ShipName);
+            n0.Replace(_T("@"), _T(" "));
+            n0.Trim();
+            wxString mmsi = wxString::Format(wxT("%i "),td->MMSI);
+            wxString n = _T("\"") + n0 + _T("\" ") + mmsi;
+            n.append(wxDateTime::Now().Format(wxT("%H:%M"))); 
+            //wxString n =  wxString::Format(wxT("\"%s\"  %i "),td->ShipName,  td->MMSI).append(wxDateTime::Now().Format(wxT("%H:%M")));
             RoutePoint *pWP = new RoutePoint( td->Lat, td->Lon, g_default_wp_icon, n, wxEmptyString );
             pWP->m_bIsolatedMark = true;                      // This is an isolated mark
             pSelect->AddSelectableRoutePoint( td->Lat, td->Lon, pWP );
@@ -242,27 +257,9 @@ void AISTargetQueryDialog::SetMMSI(int mmsi)
 
 void AISTargetQueryDialog::RecalculateSize()
 {
-    // Reset all the required member elements to cause a resize on next display
-    
-    //Set the maximum size of the entire settings dialog
-    wxSize sz = g_Platform->getDisplaySize();
-    SetSizeHints( 50, 50, sz.x-20, sz.y-40 );
-
-    m_bsize_set = false;
-
-    wxFont *dFont = FontMgr::Get().GetFont( _("AISTargetQuery") );
-    int font_size = wxMax(8, dFont->GetPointSize());
-    wxString face = dFont->GetFaceName();
-    #ifdef __WXGTK__
-    face = _T("Monospace");
-    #endif
-    m_basefont = FontMgr::Get().FindOrCreateFont( font_size, wxFONTFAMILY_MODERN,
-                                                  wxFONTSTYLE_NORMAL, dFont->GetWeight(), false, face );
-    
-    SetFont( *m_basefont );
-    m_adjustedFontSize = dFont->GetPointSize();
-    m_control_font_size = dFont->GetPointSize();
-    
+    AIS_Target_Data *td = g_pAIS->Get_Target_Data_From_MMSI( m_MMSI );
+    AdjustBestSize( td );
+    return;
 }
 
 
@@ -463,11 +460,15 @@ void AISTargetQueryDialog::AdjustBestSize( AIS_Target_Data *td )
         yb = m_createWptBtn->GetSize().y * 4;
     
     wxSize szyv = m_pQueryTextCtl->GetVirtualSize();
-    int csz = g_Platform->getDisplaySize().y * 8 / 10;
+    int csz = g_Platform->getDisplaySize().y * 85 / 100;
     if((szyv.y + yb) < csz){
         if(szyv.y > m_pQueryTextCtl->GetSize().y)
             target_y = (szyv.y * 11 / 10) + yb;
     }
+    else{                       // Probably going to be a vertical scroll bar, so adjust width slightly
+        target_y = csz;
+        target_x = szyv.x * 11/10;
+    }
 
     
     SetSize(target_x, target_y);
