@@ -32,6 +32,7 @@
 #include "routeprintout.h"
 #include "chcanv.h"
 #include "tcmgr.h"
+#include "ocpn_plugin.h"
 
 #define ID_RCLK_MENU_COPY_TEXT 7013
 #define ID_RCLK_MENU_EDIT_WP   7014
@@ -44,7 +45,6 @@ extern wxString GetLayerName(int id);
 
 extern double gLat;
 extern double gLon;
-extern MarkInfoDlg *g_pMarkInfoDialog;
 extern WayPointman *pWayPointMan;
 extern Routeman *g_pRouteMan;
 extern MyConfig *pConfig;
@@ -77,7 +77,7 @@ int g_route_prop_x, g_route_prop_y, g_route_prop_sx, g_route_prop_sy;
 #define    EVTWILIGHT    5
 #define    NIGHT        6
 
-wxString GetDaylightString(int index)
+static wxString GetDaylightString(int index)
 {
     switch (index)
     {
@@ -127,7 +127,7 @@ static double FNrange( double x )
     return ( a );
 }
 
-double getDaylightEvent( double glat, double glong, int riset, double altitude, int y, int m,
+static double getDaylightEvent( double glat, double glong, int riset, double altitude, int y, int m,
                         int d )
 {
     double day = FNday( y, m, d, 0 );
@@ -176,7 +176,7 @@ static double getLMT( double ut, double lon )
         return ( t + 24. );
 }
 
-int getDaylightStatus( double lat, double lon, wxDateTime utcDateTime )
+static int getDaylightStatus( double lat, double lon, wxDateTime utcDateTime )
 {
     if( fabs( lat ) > 60. ) return ( 0 );
     int y = utcDateTime.GetYear();
@@ -238,6 +238,7 @@ RoutePropDlgImpl::RoutePropDlgImpl( wxWindow* parent, wxWindowID id, const wxStr
     if(g_route_prop_x > 0 && g_route_prop_y > 0 && g_route_prop_x < wxGetDisplaySize().x && g_route_prop_y < wxGetDisplaySize().y) {
         SetPosition(wxPoint(10,10));
     }
+    RecalculateSize();
     
     Connect( wxEVT_COMMAND_MENU_SELECTED,
         wxCommandEventHandler(RoutePropDlgImpl::OnRoutePropMenuSelected), NULL, this );
@@ -251,6 +252,8 @@ RoutePropDlgImpl::~RoutePropDlgImpl()
 }
 
 bool RoutePropDlgImpl::instanceFlag = false;
+bool RoutePropDlgImpl::getInstanceFlag(){ return  RoutePropDlgImpl::instanceFlag; }
+
 RoutePropDlgImpl* RoutePropDlgImpl::single = NULL;
 RoutePropDlgImpl* RoutePropDlgImpl::getInstance( wxWindow* parent )
 {
@@ -258,14 +261,31 @@ RoutePropDlgImpl* RoutePropDlgImpl::getInstance( wxWindow* parent )
     {
         single = new RoutePropDlgImpl( parent );
         instanceFlag = true;
-        return single;
-    }
-    else
-    {
-        return single;
     }
+    return single;
 }
 
+void RoutePropDlgImpl::RecalculateSize(void) {
+
+  wxSize esize;
+  esize.x = GetCharWidth() * 110;
+  esize.y = GetCharHeight() * 40;
+
+  wxSize dsize = GetParent()->GetSize();  // GetClientSize();
+  esize.y = wxMin(esize.y, dsize.y - 0 /*(2 * GetCharHeight())*/);
+  esize.x = wxMin(esize.x, dsize.x - 0 /*(2 * GetCharHeight())*/);
+  SetSize(esize);
+
+  wxSize fsize = GetSize();
+  wxSize canvas_size = GetParent()->GetSize();
+  wxPoint screen_pos = GetParent()->GetScreenPosition();
+  int xp = (canvas_size.x - fsize.x) / 2;
+  int yp = (canvas_size.y - fsize.y) / 2;
+  Move(screen_pos.x + xp, screen_pos.y + yp);
+  
+}
+
+
 void RoutePropDlgImpl::UpdatePoints()
 {
     if( !m_pRoute )
@@ -274,8 +294,6 @@ void RoutePropDlgImpl::UpdatePoints()
     int selected_row = m_dvlcWaypoints->GetSelectedRow();
     m_dvlcWaypoints->DeleteAllItems();
     
-    if( NULL == m_pRoute )
-        return;
     wxVector<wxVariant> data;
     
     m_pRoute->UpdateSegmentDistances( m_pRoute->m_PlannedSpeed );           // to fix ETA properties
@@ -340,15 +358,26 @@ void RoutePropDlgImpl::UpdatePoints()
             crs = _("Arrived");
         }
 
-        data.push_back( wxVariant(in == 0 ? "---" : std::to_string(in)) );
-        data.push_back( wxVariant(name) ); // To
+        if(in == 0)
+            data.push_back( wxVariant("---"));
+        else{
+            std::ostringstream stm ;
+            stm << in ;
+            data.push_back( wxVariant(stm.str()));
+        }
+
+        wxString schar = wxEmptyString;
+#ifdef __OCPN__ANDROID__        
+        schar = wxString(" ");
+#endif        
+        data.push_back( wxVariant(name + schar) ); // To
         slen.Printf( wxT("%5.1f ") + getUsrDistanceUnit(), toUsrDistance(distance) );
-        data.push_back( wxVariant(slen) ); // Distance
-        data.push_back( wxVariant(formatAngle(bearing)) ); // Bearing
-        data.push_back( wxVariant(::toSDMM( 1, lat, FALSE)) ); // Lat
-        data.push_back( wxVariant(::toSDMM( 2, lon, FALSE)) ); // Lon
-        data.push_back( wxVariant(ete) ); // ETE
-        data.push_back( eta ); //ETA
+        data.push_back( wxVariant(schar + slen + schar) ); // Distance
+        data.push_back( wxVariant(schar + formatAngle(bearing)) ); // Bearing
+        data.push_back( wxVariant(schar + ::toSDMM( 1, lat, FALSE) + schar) ); // Lat
+        data.push_back( wxVariant(schar + ::toSDMM( 2, lon, FALSE) + schar) ); // Lon
+        data.push_back( wxVariant(schar + ete + schar) ); // ETE
+        data.push_back( schar + eta + schar ); //ETA
         data.push_back( wxVariant(wxString::FromDouble(toUsrSpeed(speed))) ); // Speed
         data.push_back( wxVariant(MakeTideInfo(tide_station, lat, lon, eta_dt)) ); // Next Tide event
         data.push_back( wxVariant(desc) ); // Description
@@ -435,15 +464,11 @@ void RoutePropDlgImpl::SetRouteAndUpdate( Route *pR, bool only_points )
     //  Fetch any config file values
     if ( !only_points )
     {
-        m_tz_selection = 1; // Local PC time by default
+        if( !pR->m_PlannedDeparture.IsValid() )
+            pR->m_PlannedDeparture = wxDateTime::Now().ToUTC();
         
-        if( pR == m_pRoute ) {
-            if( !pR->m_PlannedDeparture.IsValid() )
-                pR->m_PlannedDeparture = wxDateTime::Now();
-        } else {
-            if( !pR->m_PlannedDeparture.IsValid() )
-                pR->m_PlannedDeparture = wxDateTime::Now();
-
+        m_tz_selection = 1; // Local PC time by default
+        if( pR != m_pRoute ) {
             if( pR->m_TimeDisplayFormat == RTE_TIME_DISP_UTC)
                 m_tz_selection = 0;
             else if( pR->m_TimeDisplayFormat == RTE_TIME_DISP_LOCAL )
@@ -502,7 +527,10 @@ void RoutePropDlgImpl::SetRouteAndUpdate( Route *pR, bool only_points )
         m_tcName->SetValue( m_pRoute->m_RouteNameString );
         m_tcFrom->SetValue( m_pRoute->m_RouteStartString );
         m_tcTo->SetValue( m_pRoute->m_RouteEndString );
+        m_tcDescription->SetValue( m_pRoute->m_RouteDescription );
+
         m_tcName->SetFocus();
+#ifndef __OCPN__ANDROID__        
         if( m_pRoute->m_PlannedDeparture.IsValid() && m_pRoute->m_PlannedDeparture.GetValue() > 0 ) {
             m_dpDepartureDate->SetValue(toUsrDateTime(m_pRoute->m_PlannedDeparture, m_tz_selection , m_pRoute->pRoutePointList->GetFirst()->GetData()->m_lon).GetDateOnly());
             m_tpDepartureTime->SetValue(toUsrDateTime(m_pRoute->m_PlannedDeparture, m_tz_selection , m_pRoute->pRoutePointList->GetFirst()->GetData()->m_lon));
@@ -510,7 +538,12 @@ void RoutePropDlgImpl::SetRouteAndUpdate( Route *pR, bool only_points )
             m_dpDepartureDate->SetValue(toUsrDateTime(wxDateTime::Now(), m_tz_selection , m_pRoute->pRoutePointList->GetFirst()->GetData()->m_lon).GetDateOnly());
             m_tpDepartureTime->SetValue(toUsrDateTime(wxDateTime::Now(), m_tz_selection , m_pRoute->pRoutePointList->GetFirst()->GetData()->m_lon));
         }
+#endif        
     }
+
+    m_btnSplit->Enable(false);
+    if (!m_pRoute)
+        return;
     
     if( m_pRoute->m_Colour == wxEmptyString ) {
         m_choiceColor->Select( 0 );
@@ -539,7 +572,6 @@ void RoutePropDlgImpl::SetRouteAndUpdate( Route *pR, bool only_points )
     
     UpdatePoints();
     
-    m_btnSplit->Enable(false);
     m_btnExtend->Enable(IsThisRouteExtendable());
 }
 
@@ -564,8 +596,10 @@ void RoutePropDlgImpl::DepartureTimeOnTimeChanged( wxDateEvent& event )
 void RoutePropDlgImpl::TimezoneOnChoice( wxCommandEvent& event )
 {
     m_tz_selection = m_choiceTimezone->GetSelection();
+#ifndef __OCPN__ANDROID__
     m_dpDepartureDate->SetValue(toUsrDateTime(m_pRoute->m_PlannedDeparture, m_tz_selection , m_pRoute->pRoutePointList->GetFirst()->GetData()->m_lon).GetDateOnly());
     m_tpDepartureTime->SetValue(toUsrDateTime(m_pRoute->m_PlannedDeparture, m_tz_selection , m_pRoute->pRoutePointList->GetFirst()->GetData()->m_lon));
+#endif
     UpdatePoints();
     event.Skip();
 }
@@ -601,7 +635,7 @@ void RoutePropDlgImpl::PlanSpeedOnKillFocus( wxFocusEvent& event )
     event.Skip();
 }
 
-int ev_col;
+static int ev_col;
 void RoutePropDlgImpl::WaypointsOnDataViewListCtrlItemEditingDone( wxDataViewEvent& event )
 {
     //There is a bug in wxWidgets, the EDITING_DONE event does not contain the new value, so we must save the data and do the work later in the value changed event.
@@ -636,10 +670,15 @@ void RoutePropDlgImpl::WaypointsOnDataViewListCtrlItemValueChanged( wxDataViewEv
         ts.Trim(true);
         ts.Trim(false);
         
-        if( !etd.ParseDateTime(ts, &end) ) {
-            etd = wxInvalidDateTime;
+        if( !ts.IsEmpty() ) {
+            if( !etd.ParseDateTime(ts, &end) ) {
+                p->SetETD(wxInvalidDateTime);
+            } else {
+                p->SetETD(fromUsrDateTime(etd, m_tz_selection, p->m_lon).FormatISOCombined());
+            }
+        } else {
+            p->SetETD(wxInvalidDateTime);
         }
-        p->SetETD(fromUsrDateTime(etd, m_tz_selection, p->m_lon).FormatISOCombined());
     }
     UpdatePoints();
 #endif
@@ -647,7 +686,8 @@ void RoutePropDlgImpl::WaypointsOnDataViewListCtrlItemValueChanged( wxDataViewEv
 
 void RoutePropDlgImpl::WaypointsOnDataViewListCtrlSelectionChanged( wxDataViewEvent& event )
 {
-    if( m_dvlcWaypoints->GetSelectedRow() > 0 && m_dvlcWaypoints->GetSelectedRow() < m_dvlcWaypoints->GetItemCount() - 1 ) {
+    long selected_row = m_dvlcWaypoints->GetSelectedRow();
+    if( selected_row > 0 && selected_row < m_dvlcWaypoints->GetItemCount() - 1 ) {
         m_btnSplit->Enable(true);
     } else {
         m_btnSplit->Enable(false);
@@ -657,16 +697,30 @@ void RoutePropDlgImpl::WaypointsOnDataViewListCtrlSelectionChanged( wxDataViewEv
     } else {
         m_btnExtend->Enable(false);
     }
+    if( selected_row >= 0 && selected_row < m_dvlcWaypoints->GetItemCount()) {
+        RoutePoint *prp = m_pRoute->GetPoint(selected_row + 1);
+        if( prp ) {
+            gFrame->JumpToPosition( gFrame->GetPrimaryCanvas(), prp->m_lat, prp->m_lon, gFrame->GetPrimaryCanvas()->GetVPScale() );
+#ifdef __WXMSW__
+            if(m_dvlcWaypoints)
+                m_dvlcWaypoints->SetFocus();
+#endif
+        }
+    }
 }
 
 
 wxDateTime RoutePropDlgImpl::GetDepartureTS()
 {
+#ifndef __OCPN__ANDROID__
     wxDateTime dt = m_dpDepartureDate->GetValue();
     dt.SetHour(m_tpDepartureTime->GetValue().GetHour());
     dt.SetMinute(m_tpDepartureTime->GetValue().GetMinute());
     dt.SetSecond(m_tpDepartureTime->GetValue().GetSecond());
     return fromUsrDateTime(dt, m_tz_selection, m_pRoute->pRoutePointList->GetFirst()->GetData()->m_lon);;
+#else
+    return wxDateTime::Now();
+#endif    
 }
 
 
@@ -794,6 +848,7 @@ void RoutePropDlgImpl::SaveChanges()
         m_pRoute->m_RouteNameString = m_tcName->GetValue();
         m_pRoute->m_RouteStartString = m_tcFrom->GetValue();
         m_pRoute->m_RouteEndString = m_tcTo->GetValue();
+        m_pRoute->m_RouteDescription  = m_tcDescription->GetValue();
         if( m_choiceColor->GetSelection() == 0 ) {
             m_pRoute->m_Colour = wxEmptyString;
         } else {
