@@ -85,12 +85,13 @@ millions of points.
 #include "Select.h"
 #include "chcanv.h"
 
+#include "pluginmanager.h"
+
 #ifdef ocpnUSE_GL
 #include "glChartCanvas.h"
 extern ocpnGLOptions g_GLOptions;
 #endif
 
-extern MyFrame *gFrame;
 extern WayPointman *pWayPointMan;
 extern Routeman *g_pRouteMan;
 extern Select *pSelect;
@@ -103,6 +104,7 @@ extern bool             g_bHighliteTracks;
 extern double           g_TrackDeltaDistance;
 extern float            g_GLMinSymbolLineWidth;
 extern wxColour         g_colourTrackLineColour;
+extern PlugInManager    *g_pi_manager;
 extern wxColor GetDimColor(wxColor c);
 
 #if defined( __UNIX__ ) && !defined(__WXOSX__)  // high resolution stopwatch for profiling
@@ -622,8 +624,6 @@ void Track::Draw( ChartCanvas *cc, ocpnDC& dc, ViewPort &VP, const LLBBox &box )
     if(!pointlists.size())
         return;
 
-    unsigned short int FromSegNo = 1;
-
     //  Establish basic colour
     wxColour basic_colour;
     if( IsRunning() )
@@ -657,7 +657,12 @@ void Track::Draw( ChartCanvas *cc, ocpnDC& dc, ViewPort &VP, const LLBBox &box )
             radius = 0;
     }
 
-    if( dc.GetDC() || radius ) {
+#ifndef USE_ANDROID_GLES2
+    if(dc.GetDC() || radius)
+#else
+    if(1)
+#endif    
+    {
         dc.SetPen( *wxThePenList->FindOrCreatePen( col, width, style ) );
         dc.SetBrush( *wxTheBrushList->FindOrCreateBrush( col, wxBRUSHSTYLE_SOLID ) );
         for(std::list< std::list<wxPoint> >::iterator lines = pointlists.begin();
@@ -672,7 +677,7 @@ void Track::Draw( ChartCanvas *cc, ocpnDC& dc, ViewPort &VP, const LLBBox &box )
             }
 
             int hilite_width = radius;
-            if( hilite_width ) {
+            if( hilite_width >= 1.0 ) {
                 wxPen psave = dc.GetPen();
 
                 dc.StrokeLines( i, points );
@@ -693,6 +698,7 @@ void Track::Draw( ChartCanvas *cc, ocpnDC& dc, ViewPort &VP, const LLBBox &box )
         }
     }
 #ifdef ocpnUSE_GL    
+#ifndef USE_ANDROID_GLES2
     else { // opengl version
         glColor3ub(col.Red(), col.Green(), col.Blue());
         glLineWidth( wxMax( g_GLMinSymbolLineWidth, width ) );
@@ -721,7 +727,8 @@ void Track::Draw( ChartCanvas *cc, ocpnDC& dc, ViewPort &VP, const LLBBox &box )
                 i+=2;
             }
 
-            glDrawArrays(GL_LINE_STRIP, 0, i >> 1);
+            if(i > 2)
+                glDrawArrays(GL_LINE_STRIP, 0, i >> 1);
         }
         glDisableClientState(GL_VERTEX_ARRAY);
 
@@ -731,7 +738,7 @@ void Track::Draw( ChartCanvas *cc, ocpnDC& dc, ViewPort &VP, const LLBBox &box )
         
     }
 #endif
-
+#endif
     if(m_HighlightedTrackPoint >= 0)
         TrackPoints[m_HighlightedTrackPoint]->Draw(cc, dc);
 }
@@ -972,6 +979,16 @@ TrackPoint* Track::AddNewPoint( vector2D point, wxDateTime time )
     AddPointFinalized( tPoint );
 
     pConfig->AddNewTrackPoint( tPoint, m_GUID );        // This will update the "changes" file only
+
+    //send a wxJson message to all plugins
+    wxJSONValue v;
+    v[_T("lat")] = tPoint->m_lat;
+    v[_T("lon")] = tPoint->m_lon;
+    v[_T("Track_ID")] = m_GUID;
+    wxString msg_id( _T("OCPN_TRK_POINT_ADDED") );
+    g_pi_manager->SendJSONMessageToAllPlugins( msg_id, v );
+    //
+
     return tPoint;
 }
 
@@ -1090,7 +1107,7 @@ Route *Track::RouteFromTrack( wxGenericProgressDialog *pprog )
     pSelect->AddSelectableRoutePoint( pWP_dst->m_lat, pWP_dst->m_lon, pWP_dst );
     pWP_prev = pWP_dst;
 // add intermediate points as needed
-
+    int dProg = 0;
     for(size_t i = 1; i < TrackPoints.size();) {
         TrackPoint *prp = TrackPoints[i];
         prpnodeX = i;
@@ -1176,7 +1193,12 @@ Route *Track::RouteFromTrack( wxGenericProgressDialog *pprog )
             i++;
             next_ic++;
         }
-        if( pprog ) pprog->Update( ( i * 100 ) / nPoints );
+        int iProg = (i * 100) / nPoints;
+        if (pprog && (iProg > dProg))
+        {
+            dProg = iProg;
+            pprog->Update(dProg);
+        }
     }
 
 // add last point, if needed
