@@ -35,6 +35,7 @@
 #include <wx/clipbrd.h>
 #include <wx/aui/aui.h>
 
+#include "config.h"
 #include "dychart.h"
 
 #include <wx/listimpl.cpp>
@@ -67,11 +68,9 @@
 #include "Track.h"
 #include "Route.h"
 
-#ifdef USE_S57
 #include "cm93.h"                   // for chart outline draw
 #include "s57chart.h"               // for ArrayOfS57Obj
 #include "s52plib.h"
-#endif
 
 #include "ais.h"
 
@@ -102,33 +101,28 @@ extern int              g_nAWMax;
 extern int              g_nAWDefault;
 extern RoutePoint       *pAnchorWatchPoint1;
 extern RoutePoint       *pAnchorWatchPoint2;
-extern double           AnchorPointMinDist;
-extern bool             AnchorAlertOn1;
-extern bool             AnchorAlertOn2;
 extern wxString         g_AW1GUID;
 extern wxString         g_AW2GUID;
 extern int              g_click_stop;
 extern RouteManagerDialog *pRouteManagerDialog;
 extern MarkInfoDlg     *g_pMarkInfoDialog;
 extern RoutePropDlgImpl *pRoutePropDialog;
-extern TrackPropDlg     *pTrackPropDialog;
 extern ActiveTrack      *g_pActiveTrack;
 extern bool             g_bConfirmObjectDelete;
 extern WayPointman      *pWayPointMan;
 extern MyConfig         *pConfig;
 extern Select           *pSelect;
 
-#ifdef USE_S57
-extern s52plib          *ps52plib;
 extern CM93OffsetDialog  *g_pCM93OffsetDialog;
-#endif
 
 extern GoToPositionDialog *pGoToPositionDialog;
 extern RouteList        *pRouteList;
 extern wxString         g_default_wp_icon;
 extern bool              g_btouch;
 extern bool             g_bBasicMenus;
-
+extern TrackPropDlg     *pTrackPropDialog;
+extern double           gHdt;
+extern bool             g_FlushNavobjChanges;
 
 
 //    Constants for right click menus
@@ -197,6 +191,7 @@ enum
     ID_DEF_MENU_QUILTREMOVE,
     ID_DEF_MENU_COGUP,
     ID_DEF_MENU_NORTHUP,
+    ID_DEF_MENU_HEADUP,
     ID_DEF_MENU_TOGGLE_FULL,
     ID_DEF_MENU_TIDEINFO,
     ID_DEF_MENU_CURRENTINFO,
@@ -256,6 +251,7 @@ void MenuPrepend1( wxMenu *menu, int id, wxString label)
     item->SetFont(sFont);
 #endif
     
+    if(g_btouch) menu->InsertSeparator(0);
     menu->Prepend(item);
 }
 
@@ -274,6 +270,7 @@ void MenuAppend1( wxMenu *menu, int id, wxString label)
 #endif
     
     menu->Append(item);
+    if(g_btouch) menu->AppendSeparator();
 }
 
 void SetMenuItemFont1(wxMenuItem *item)
@@ -422,10 +419,18 @@ if( !g_bBasicMenus && (nChartStack > 1 ) ) {
         MenuAppend1( contextMenu, ID_DEF_MENU_GOTOPOSITION, _("Center view") + _T("...") );
 
     if( !g_bBasicMenus){
-        if( !parent->m_bCourseUp )
-            MenuAppend1( contextMenu, ID_DEF_MENU_COGUP, _("Course Up Mode") );
+        if( parent->GetVP().b_quilt){
+            if( parent->GetUpMode() == NORTH_UP_MODE ){
+                MenuAppend1( contextMenu, ID_DEF_MENU_COGUP, _("Course Up Mode") );
+                if(!std::isnan(gHdt))
+                    MenuAppend1( contextMenu, ID_DEF_MENU_HEADUP, _("Heading Up Mode") );
+            }
+            else{
+                MenuAppend1( contextMenu, ID_DEF_MENU_NORTHUP, _("North Up Mode") );
+            }
+        }    
         else {
-            if( !parent->GetVP().b_quilt && parent->m_singleChart && ( fabs( parent->m_singleChart->GetChartSkew() ) > .01 )
+            if( parent->m_singleChart && ( fabs( parent->m_singleChart->GetChartSkew() ) > .01 )
                 && !g_bskew_comp ) MenuAppend1( contextMenu, ID_DEF_MENU_NORTHUP, _("Chart Up Mode") );
             else
                 MenuAppend1( contextMenu, ID_DEF_MENU_NORTHUP, _("North Up Mode") );
@@ -500,6 +505,8 @@ if( !g_bBasicMenus && (nChartStack > 1 ) ) {
 
     //  ChartGroup SubMenu
     wxMenuItem* subItemChart = contextMenu->AppendSubMenu( subMenuChart, _("Chart Groups") );
+    if(g_btouch) contextMenu->AppendSeparator();
+    
     SetMenuItemFont1(subItemChart);
     
     if( g_pGroupArray->GetCount() ) {
@@ -634,7 +641,7 @@ if( !g_bBasicMenus && (nChartStack > 1 ) ) {
             MenuAppend1( menuRoute, ID_RT_MENU_DELETE, _( "Delete" ) + _T( "..." ) );
             MenuAppend1( menuRoute, ID_RT_MENU_REVERSE, _( "Reverse..." ) );
 
-#ifndef __OCPN__ANDROID__
+//#ifndef __OCPN__ANDROID__
             wxString port = parent->FindValidUploadPort();
             parent->m_active_upload_port = port;
             wxString item = _( "Send to GPS" );
@@ -649,7 +656,7 @@ if( !g_bBasicMenus && (nChartStack > 1 ) ) {
                 wxString item = _( "Send to new GPS" );
                 MenuAppend1( menuRoute, ID_RT_MENU_SENDTONEWGPS, item );
             }
-#endif                
+//#endif                
 		}
         //Eventually set this menu as the "focused context menu"
 		if (menuFocus != menuAIS)
@@ -723,6 +730,7 @@ if( !g_bBasicMenus && (nChartStack > 1 ) ) {
             if( m_pFoundRoutePoint && m_pFoundRoutePoint->GetIconName() != _T("mob") )
                 MenuAppend1( menuWaypoint, ID_RT_MENU_DELPOINT,  _( "Delete" ) );
 
+//#ifndef __OCPN__ANDROID__
             wxString port = parent->FindValidUploadPort();
             parent->m_active_upload_port = port;
             wxString item = _( "Send to GPS" );
@@ -737,7 +745,7 @@ if( !g_bBasicMenus && (nChartStack > 1 ) ) {
                 wxString item = _( "Send to new GPS" );
                 MenuAppend1( menuWaypoint, ID_WPT_MENU_SENDTONEWGPS, item );
             }
-            
+//#endif            
             
         }
         //Eventually set this menu as the "focused context menu"
@@ -773,6 +781,7 @@ if( !g_bBasicMenus && (nChartStack > 1 ) ) {
             if( m_pFoundRoutePoint && m_pFoundRoutePoint->GetIconName() != _T("mob") )
                 MenuAppend1( menuWaypoint, ID_WP_MENU_DELPOINT, _( "Delete" ) );
 
+//#ifndef __OCPN__ANDROID__            
             wxString port = parent->FindValidUploadPort();
             parent->m_active_upload_port = port;
             wxString item = _( "Send to GPS" );
@@ -782,7 +791,7 @@ if( !g_bBasicMenus && (nChartStack > 1 ) ) {
                 item.Append(_T(" )") );
             }
             MenuAppend1( menuWaypoint, ID_WPT_MENU_SENDTOGPS, item );
-
+//#endif
 
             if( ( m_pFoundRoutePoint == pAnchorWatchPoint1 ) || ( m_pFoundRoutePoint == pAnchorWatchPoint2 ) )
                 MenuAppend1( menuWaypoint, ID_WP_MENU_CLEAR_ANCHORWATCH, _( "Clear Anchor Watch" ) );
@@ -919,8 +928,24 @@ if( !g_bBasicMenus && (nChartStack > 1 ) ) {
     }
 
     //        Invoke the correct focused drop-down menu
+    
+#ifdef __OCPN__ANDROID__
+    androidEnableBackButton( false );
+    androidEnableOptionsMenu( false );
+
+    setMenuStyleSheet(menuRoute, GetOCPNGUIScaledFont(_T("Menu")));
+    setMenuStyleSheet(menuWaypoint, GetOCPNGUIScaledFont(_T("Menu")));
+    setMenuStyleSheet(menuTrack, GetOCPNGUIScaledFont(_T("Menu")));
+    setMenuStyleSheet(menuAIS, GetOCPNGUIScaledFont(_T("Menu")));
+#endif
+    
+    
     parent->PopupMenu( menuFocus, x, y );
 
+#ifdef __OCPN__ANDROID__
+    androidEnableBackButton( true );
+    androidEnableOptionsMenu( true );
+#endif
 
     /* Cleanup if necessary.
 	Do not delete menus witch are submenu as they will be deleted by their parent menu.
@@ -956,6 +981,7 @@ void CanvasMenuHandler::PopupMenuHandler( wxCommandEvent& event )
 
     case ID_DEF_MENU_SCALE_IN:
         parent->DoCanvasStackDelta( -1 );
+	break;
 
     case ID_DEF_MENU_SCALE_OUT:
         parent->DoCanvasStackDelta( 1 );
@@ -1018,11 +1044,17 @@ void CanvasMenuHandler::PopupMenuHandler( wxCommandEvent& event )
         pConfig->AddNewWayPoint( pWP, -1 );    // use auto next num
         if( !pWP->IsVisibleSelectable(this->parent) ) pWP->ShowScaleWarningMessage(parent);
 
-        if( pRouteManagerDialog && pRouteManagerDialog->IsShown() ) pRouteManagerDialog->UpdateWptListCtrl();
+        if(RouteManagerDialog::getInstanceFlag()){
+            if( pRouteManagerDialog && pRouteManagerDialog->IsShown() ){
+                pRouteManagerDialog->UpdateWptListCtrl();
+            }
+        }
+        
         parent->undo->BeforeUndoableAction( Undo_CreateWaypoint, pWP, Undo_HasParent, NULL );
         parent->undo->AfterUndoableAction( NULL );
         gFrame->RefreshAllCanvas( false );
         gFrame->InvalidateAllGL();
+        g_FlushNavobjChanges = true;
         break;
     }
 
@@ -1068,11 +1100,15 @@ void CanvasMenuHandler::PopupMenuHandler( wxCommandEvent& event )
     }
 
     case ID_DEF_MENU_COGUP:
-        parent->ToggleCourseUp();
+        parent->SetUpMode(COURSE_UP_MODE);
+        break;
+
+   case ID_DEF_MENU_HEADUP:
+        parent->SetUpMode(HEAD_UP_MODE);
         break;
 
     case ID_DEF_MENU_NORTHUP:
-        parent->ToggleCourseUp();
+        parent->SetUpMode(NORTH_UP_MODE);
         break;
         
     case ID_DEF_MENU_TOGGLE_FULL:
@@ -1119,8 +1155,10 @@ void CanvasMenuHandler::PopupMenuHandler( wxCommandEvent& event )
                 g_pMarkInfoDialog->UpdateProperties();
             }
 
-            if( pRouteManagerDialog && pRouteManagerDialog->IsShown() )
+            if( pRouteManagerDialog ){
+                if( pRouteManagerDialog->IsShown() )
                 pRouteManagerDialog->UpdateWptListCtrl();
+            }
 
             gFrame->RefreshAllCanvas( false );
             gFrame->InvalidateAllGL();
@@ -1174,7 +1212,6 @@ void CanvasMenuHandler::PopupMenuHandler( wxCommandEvent& event )
         parent->Refresh( false );
         break;
 
-#ifdef USE_S57
     case ID_DEF_MENU_CM93OFFSET_DIALOG:
     {
         if( NULL == g_pCM93OffsetDialog ) {
@@ -1198,7 +1235,6 @@ void CanvasMenuHandler::PopupMenuHandler( wxCommandEvent& event )
         parent->ShowObjectQueryWindow( popx, popy, zlat, zlon );
         break;
     }
-#endif
     case ID_DEF_MENU_AIS_QUERY: {
         wxWindow *pwin = wxDynamicCast(parent, wxWindow);
         ShowAISTargetQueryDialog( pwin, m_FoundAIS_MMSI );
@@ -1416,6 +1452,8 @@ void CanvasMenuHandler::PopupMenuHandler( wxCommandEvent& event )
              else {
                  SendToGpsDlg dlg;
                  dlg.SetWaypoint( m_pFoundRoutePoint );
+                 wxFont fo = GetOCPNGUIScaledFont(_T("Dialog"));
+                 dlg.SetFont(fo);
 
                  dlg.Create( NULL, -1, _( "Send to GPS" ) + _T( "..." ), _T("") );
                  dlg.ShowModal();
@@ -1550,7 +1588,12 @@ void CanvasMenuHandler::PopupMenuHandler( wxCommandEvent& event )
             pConfig->DeleteConfigTrack( m_pSelectedTrack );
             g_pRouteMan->DeleteTrack( m_pSelectedTrack );
 
-            if( pRouteManagerDialog && pRouteManagerDialog->IsShown() ) {
+            if( TrackPropDlg::getInstanceFlag() && pTrackPropDialog &&
+                ( pTrackPropDialog->IsShown()) && (m_pSelectedTrack == pTrackPropDialog->GetTrack()) ) {
+                pTrackPropDialog->Hide();
+            }
+
+            if( RoutePropDlgImpl::getInstanceFlag() && pRouteManagerDialog && pRouteManagerDialog->IsShown() ) {
                 pRouteManagerDialog->UpdateTrkListCtrl();
                 pRouteManagerDialog->UpdateRouteListCtrl();
             }
@@ -1588,6 +1631,7 @@ void CanvasMenuHandler::PopupMenuHandler( wxCommandEvent& event )
         parent->FinishRoute();
         gFrame->SurfaceAllCanvasToolbars();
         parent->Refresh( false );
+        g_FlushNavobjChanges = true;
         break;
 
     case ID_DEF_ZERO_XTE:
