@@ -46,10 +46,14 @@
 #define UINT32 unsigned int
 #endif
 
+#ifdef __OCPN__ANDROID__
+#include "androidUTIL.h"
+#endif
+
 extern PlugInManager    *g_pi_manager;
 extern wxString         gWorldMapLocation;
 
-int s_dbVersion;                                //    Database version currently in use at runtime
+static int s_dbVersion;                         //    Database version currently in use at runtime
                                                 //  Needed for ChartTableEntry::GetChartType() only
                                                 //  TODO This can go away at opencpn Version 1.3.8 and above....
 ///////////////////////////////////////////////////////////////////////
@@ -64,7 +68,7 @@ bool FindMatchingFile(const wxString &theDir, const wxChar *theRegEx, int nameLe
 }
 
 
-ChartFamilyEnum GetChartFamily(int charttype)
+static ChartFamilyEnum GetChartFamily(int charttype)
 {
       ChartFamilyEnum cf;
 
@@ -157,12 +161,12 @@ void ChartTableEntry::SetScale( int scale )
        rounding = 5 *pow(10, log10(Scale) -2);
 }
 
-ChartTableEntry::ChartTableEntry(ChartBase &theChart)
+ChartTableEntry::ChartTableEntry(ChartBase &theChart, wxString& utf8Path)
 {
     Clear();
 
-    char *pt = (char *)malloc(strlen(theChart.GetFullPath().mb_str(wxConvUTF8)) + 1);
-    strcpy(pt, theChart.GetFullPath().mb_str(wxConvUTF8));
+    char *pt = (char *)malloc(strlen(utf8Path.mb_str(wxConvUTF8)) + 1);
+    strcpy(pt, utf8Path.mb_str(wxConvUTF8));
     pFullPath = pt;
 
 
@@ -185,7 +189,12 @@ ChartTableEntry::ChartTableEntry(ChartBase &theChart)
     m_pfilename = new wxString;           // create and populate helper members
     *m_pfilename = fn.GetFullName();
     m_psFullPath = new wxString;
-    *m_psFullPath = theChart.GetFullPath();
+    *m_psFullPath = utf8Path;
+    m_fullSystemPath = utf8Path;
+        
+#ifdef __OCPN__ANDROID__
+        m_fullSystemPath = wxString(utf8Path.mb_str(wxConvUTF8));
+#endif       
     
     Extent ext;
     theChart.GetChartExtent(&ext);
@@ -398,24 +407,32 @@ ChartTableEntry::~ChartTableEntry()
 bool ChartTableEntry::IsEarlierThan(const ChartTableEntry &cte) const {
     wxDateTime mine(edition_date);
     wxDateTime theirs(cte.edition_date);
+    
+    if(!mine.IsValid() || !theirs.IsValid())
+        return false;              // will have the effect of keeping all questionable charts                     
+    
     return ( mine.IsEarlierThan(theirs) );
 }
 
 bool ChartTableEntry::IsEqualTo(const ChartTableEntry &cte) const {
       wxDateTime mine(edition_date);
       wxDateTime theirs(cte.edition_date);
+      
+      if(!mine.IsValid() || !theirs.IsValid())
+          return true;              // will have the effect of keeping all questionable charts                     
+      
       return ( mine.IsEqualTo(theirs) );
 }
 
 ///////////////////////////////////////////////////////////////////////
 
-int ChartTableEntry::GetChartType() const
+static int convertChartType(int charttype)
 {
       //    Hackeroo here....
       //    dB version 14 had different ChartType Enum, patch it here
       if(s_dbVersion == 14)
       {
-            switch(ChartType)
+            switch(charttype)
             {
                   case 0: return CHART_TYPE_KAP;
                   case 1: return CHART_TYPE_GEO;
@@ -428,15 +445,14 @@ int ChartTableEntry::GetChartType() const
                   default: return CHART_TYPE_UNKNOWN;
             }
       }
-      else
-       return ChartType;
+      return charttype;
 }
 
-int ChartTableEntry::GetChartFamily() const
+static int convertChartFamily(int charttype, int chartfamily)
 {
     if(s_dbVersion < 18)
     {
-        switch(ChartType)
+        switch(charttype)
         {
             case CHART_TYPE_KAP:
             case CHART_TYPE_GEO:
@@ -451,13 +467,10 @@ int ChartTableEntry::GetChartFamily() const
                   return CHART_FAMILY_UNKNOWN;
       }
     }
-    else
-        return ChartFamily;
+    return chartfamily;
 }
 
 
-
-
 bool ChartTableEntry::Read(const ChartDatabase *pDb, wxInputStream &is)
 {
     char path[4096], *cp;
@@ -483,7 +496,11 @@ bool ChartTableEntry::Read(const ChartDatabase *pDb, wxInputStream &is)
         *m_pfilename = fn.GetFullName();
         m_psFullPath = new wxString;
         *m_psFullPath = fullfilename;
+        m_fullSystemPath = fullfilename;
         
+#ifdef __OCPN__ANDROID__
+        m_fullSystemPath = wxString(fullfilename.mb_str(wxConvUTF8));
+#endif       
         // Read the table entry
         ChartTableEntry_onDisk_18 cte;
         is.Read(&cte, sizeof(ChartTableEntry_onDisk_18));
@@ -785,6 +802,8 @@ bool ChartTableEntry::Read(const ChartDatabase *pDb, wxInputStream &is)
                 }
           }
     }
+    ChartFamily = convertChartFamily(ChartType, ChartFamily);
+    ChartType = convertChartType(ChartType);
 
     return true;
 }
@@ -1064,6 +1083,8 @@ WX_DEFINE_OBJARRAY(ArrayOfChartClassDescriptor);
 ChartDatabase::ChartDatabase()
 {
       bValid = false;
+      m_b_busy = false;
+      
       m_ChartTableEntryDummy.Clear();
 
       UpdateChartClassDescriptorArray();
@@ -1118,7 +1139,7 @@ void ChartDatabase::UpdateChartClassDescriptorArray(void)
 
 const ChartTableEntry &ChartDatabase::GetChartTableEntry(int index) const
 {
-      if(index < m_nentries)
+      if(index < GetChartTableEntries())
           return active_chartTable[index];
       else
             return m_ChartTableEntryDummy;
@@ -1126,7 +1147,7 @@ const ChartTableEntry &ChartDatabase::GetChartTableEntry(int index) const
 
 ChartTableEntry *ChartDatabase::GetpChartTableEntry(int index) const
 {
-    if(index < m_nentries)
+    if(index < GetChartTableEntries())
           return &active_chartTable[index];
       else
             return (ChartTableEntry *)&m_ChartTableEntryDummy;
@@ -1232,7 +1253,7 @@ bool ChartDatabase::Read(const wxString &filePath)
     active_chartTable.Alloc(entries);
     active_chartTable_pathindex.clear();
     while (entries-- && entry.Read(this, ifs)) {
-        active_chartTable_pathindex[wxString(entry.GetpFullPath(), wxConvUTF8)] = ind++;
+        active_chartTable_pathindex[entry.GetFullSystemPath()] = ind++;
         active_chartTable.Add(entry);
     }
 
@@ -1335,25 +1356,11 @@ wxString ChartDatabase::GetFullChartInfo(ChartBase *pc, int dbIndex, int *char_w
       {
             wxString line;
             line = _(" ChartFile:  ");
-            wxString longline(cte.GetpFullPath(), wxConvUTF8);
-/*
-            if(CHART_TYPE_CM93COMP == cte.GetChartType())
-            {
-                  //    Remove filename and two trailing directories
-                  //    to make it pretty....
-                  wxString path;
-                  const wxString full_path(cte.GetpFullPath(), wxConvUTF8);
-                  wxFileName::SplitPath(full_path, &path, NULL, NULL);
+            wxString longline = *(cte.GetpsFullPath());
 
-                  wxFileName fn(path);
-                  fn.RemoveLastDir();
-                  longline = fn.GetPath();
-                  longline += _T("...");
-            }
-*/
             if(longline.Len() > target_width)
             {
-                  line += SplitPath(wxString(cte.GetpFullPath(), wxConvUTF8), _T("/,\\"), target_width, 15, &ncr);
+                  line += SplitPath(longline, _T("/,\\"), target_width, 15, &ncr);
                   max_width = wxMax(max_width, target_width+4);
                   lc += ncr;
             }
@@ -1537,6 +1544,7 @@ bool ChartDatabase::Update(ArrayOfCDI& dir_array, bool bForce, wxGenericProgress
       m_dir_array = dir_array;
 
       bValid = false;               // database is not useable right now...
+      m_b_busy = true;
 
       //  Mark all charts provisionally invalid
       for(unsigned int i=0 ; i<active_chartTable.GetCount() ; i++)
@@ -1567,11 +1575,15 @@ bool ChartDatabase::Update(ArrayOfCDI& dir_array, bool bForce, wxGenericProgress
             ChartDirInfo dir_info = dir_array[j];
 
             wxString dir_magic;
+            
+            if(dir_info.fullpath.Find(_T("GSHHG")) != wxNOT_FOUND){
             if( !wxDir::FindFirst(dir_info.fullpath, "poly-*-1.dat").empty() ) {
             //If some polygons exist in the directory, set it as the one to use for GSHHG
             //TODO: We should probably compare the version and maybe resolutions available with what is currently used...
                 gWorldMapLocation = dir_info.fullpath + wxFileName::GetPathSeparator();
             }
+            }
+            
             TraverseDirAndAddCharts(dir_info, pprog, dir_magic, lbForce);
 
         //  Update the dir_list entry, even if the magic values are the same
@@ -1596,13 +1608,14 @@ bool ChartDatabase::Update(ArrayOfCDI& dir_array, bool bForce, wxGenericProgress
       //    And once more, setting the Entry index field
       active_chartTable_pathindex.clear();
       for(unsigned int i=0 ; i<active_chartTable.GetCount() ; i++) {
-          active_chartTable_pathindex[wxString(active_chartTable[i].GetpFullPath(), wxConvUTF8)] = i;
+          active_chartTable_pathindex[active_chartTable[i].GetFullSystemPath()] = i;
           active_chartTable[i].SetEntryOffset( i );
       }
 
       m_nentries = active_chartTable.GetCount();
       
       bValid = true;
+      m_b_busy = false;
       return true;
 }
 
@@ -1616,8 +1629,7 @@ int ChartDatabase::FinddbIndex(wxString PathToFind)
       //    Find the chart
       for(unsigned int i=0 ; i<active_chartTable.GetCount() ; i++)
       {
-          wxString s = wxString(active_chartTable[i].GetpFullPath(), wxConvUTF8);
-          if(s.IsSameAs(PathToFind))
+          if(active_chartTable[i].GetpsFullPath()->IsSameAs(PathToFind))
             {
                   return i;
             }
@@ -1657,6 +1669,10 @@ int ChartDatabase::TraverseDirAndAddCharts(ChartDirInfo& dir_info, wxGenericProg
 {
       //    Extract the true dir name and magic number from the compound string
       wxString dir_path = dir_info.fullpath;
+#ifdef __OCPN__ANDROID__
+      dir_path = wxString(dir_info.fullpath.mb_str(wxConvUTF8));
+#endif      
+      
       wxString old_magic = dir_info.magic_number;
       wxString new_magic = old_magic;
       dir_magic = old_magic;              // provisionally the same
@@ -1684,7 +1700,7 @@ int ChartDatabase::TraverseDirAndAddCharts(ChartDirInfo& dir_info, wxGenericProg
       //    Quick scan the directory to see if it has changed
       //    If not, there is no need to scan again.....
       if(!b_skipDetectDirChange)
-            b_dirchange = DetectDirChange(dir_path, old_magic, new_magic, pprog);
+            b_dirchange = DetectDirChange(dir_path, dir_info.fullpath, old_magic, new_magic, pprog);
 
       if( !bForce && !b_dirchange)
       {
@@ -1706,7 +1722,7 @@ int ChartDatabase::TraverseDirAndAddCharts(ChartDirInfo& dir_info, wxGenericProg
             for(int ic=0 ; ic<nEntries ; ic++)
             {
 
-                wxFileName fn(wxString(active_chartTable[ic].GetpFullPath(), wxConvUTF8));
+                wxFileName fn(active_chartTable[ic].GetFullSystemPath());
 
                   while(fn.GetDirCount() >= dir_path_count)
                   {
@@ -1731,13 +1747,13 @@ int ChartDatabase::TraverseDirAndAddCharts(ChartDirInfo& dir_info, wxGenericProg
       //    Look for all possible defined chart classes
       for(unsigned int i = 0 ; i < m_ChartClassDescriptorArray.GetCount() ; i++)
       {
-            nAdd += SearchDirAndAddCharts(dir_path, m_ChartClassDescriptorArray[i], pprog);
+            nAdd += SearchDirAndAddCharts(dir_info.fullpath, m_ChartClassDescriptorArray.Item(i), pprog);
       }
 
       return nAdd;
 }
 
-bool ChartDatabase::DetectDirChange(const wxString & dir_path, const wxString & magic, wxString &new_magic, wxGenericProgressDialog *pprog)
+bool ChartDatabase::DetectDirChange(const wxString & dir_path, const wxString& prog_label, const wxString & magic, wxString &new_magic, wxGenericProgressDialog *pprog)
 {
       if(pprog)
             pprog->SetTitle(_("OpenCPN Directory Scan...."));
@@ -1761,7 +1777,7 @@ bool ChartDatabase::DetectDirChange(const wxString & dir_path, const wxString &
       for(int ifile=0 ; ifile < n_files ; ifile++)
       {
             if(pprog && (ifile % (n_files / 60 + 1)) == 0)
-                  pprog->Update(wxMin((ifile * 100) /n_files, 100), dir_path);
+                  pprog->Update(wxMin((ifile * 100) /n_files, 100), prog_label);
 
             wxFileName file(FileList[ifile]);
 
@@ -1826,6 +1842,14 @@ bool ChartDatabase::Check_CM93_Structure(wxString dir_name)
       wxDir dirt(dir_name);
       wxString candidate;
 
+      if(dirt.IsOpened())
+        wxLogMessage(_T("check_cm93 opened dir OK:  ") + dir_name);
+      else{
+        wxLogMessage(_T("check_cm93 NOT OPENED OK:  ") + dir_name);
+        wxLogMessage(_T("check_cm93 returns false.") + dir_name);
+        return false;
+      }
+
       bool b_maybe_found_cm93 = false;
       bool b_cont = dirt.GetFirst(&candidate);
 
@@ -1849,6 +1873,8 @@ bool ChartDatabase::Check_CM93_Structure(wxString dir_name)
             if(wxDir::Exists(dir_next))
             {
                   wxDir dir_n(dir_next);
+                  if(dirt.IsOpened()){
+
                   wxString candidate_n;
 
                   wxRegEx test_n(_T("^[A-Ga-g]"));
@@ -1878,6 +1904,7 @@ bool ChartDatabase::Check_CM93_Structure(wxString dir_name)
                   }
             }
       }
+      }
 
       return false;
 }
@@ -2003,10 +2030,16 @@ int ChartDatabase::SearchDirAndAddCharts(wxString& dir_name_base,
       msg += chart_desc.m_search_mask;
       wxLogMessage(msg);
 
-      if(!wxDir::Exists(dir_name_base))
-            return 0;
 
       wxString dir_name = dir_name_base;
+
+#ifdef __OCPN__ANDROID__      
+      dir_name = wxString(dir_name_base.mb_str(wxConvUTF8));  // android
+#endif
+
+      if(!wxDir::Exists(dir_name))
+            return 0;
+
       wxString filespec = chart_desc.m_search_mask.Upper();
       wxString lowerFileSpec = chart_desc.m_search_mask.Lower();
       wxString filespecXZ = filespec + _T(".xz");
@@ -2040,11 +2073,36 @@ int ChartDatabase::SearchDirAndAddCharts(wxString& dir_name_base,
             // Note that `wxDir::GetAllFiles()` appends to the list rather than replaces existing contents.
             wxDir dir(dir_name);
             dir.GetAllFiles(dir_name, &FileList, filespec, gaf_flags);
+
+#ifdef __OCPN__ANDROID__
+            if(!FileList.GetCount()){
+                wxArrayString afl = androidTraverseDir( dir_name, filespec);
+
+                for (wxArrayString::const_iterator item = afl.begin(); item != afl.end(); item++)
+                    FileList.Add(*item);
+            }
+#endif
+            // add xz compressed files;
             dir.GetAllFiles(dir_name, &FileList, filespecXZ, gaf_flags);
 #ifndef __WXMSW__
             if (filespec != lowerFileSpec)
             {
-                dir.GetAllFiles(dir_name, &FileList, lowerFileSpec, gaf_flags);
+            // add lowercase filespec files too
+                wxArrayString lowerFileList;
+                dir.GetAllFiles(dir_name, &lowerFileList, lowerFileSpec, gaf_flags);
+
+#ifdef __OCPN__ANDROID__
+                if(!lowerFileList.GetCount()){
+                    wxArrayString afl = androidTraverseDir( dir_name, lowerFileSpec);
+
+                    for (wxArrayString::const_iterator item = afl.begin(); item != afl.end(); item++)
+                        lowerFileList.Add(*item);
+                }
+#endif
+
+                for (wxArrayString::const_iterator item = lowerFileList.begin(); item != lowerFileList.end(); item++)
+                    FileList.Add(*item);
+
                 dir.GetAllFiles(dir_name, &FileList, lowerFileSpecXZ, gaf_flags);
             }
 #endif
@@ -2078,7 +2136,7 @@ int ChartDatabase::SearchDirAndAddCharts(wxString& dir_name_base,
       ChartCollisionsHashMap collision_map; 
       int nEntry = active_chartTable.GetCount();
       for(int i=0 ; i<nEntry ; i++) {
-          wxString table_file_name(active_chartTable[i].GetpFullPath(), wxConvUTF8);
+          wxString table_file_name = active_chartTable[i].GetFullSystemPath();
           wxFileName table_file(table_file_name);
           collision_map[table_file.GetFullName()] = i;
       }
@@ -2091,6 +2149,25 @@ int ChartDatabase::SearchDirAndAddCharts(wxString& dir_name_base,
             wxFileName file(FileList[ifile]);
             wxString full_name = file.GetFullPath();
             wxString file_name = file.GetFullName();
+            wxString utf8_path = full_name;
+            
+#ifdef __OCPN__ANDROID__
+            // The full path (full_name) is the broken Android files system interpretation, which does not display well onscreen.
+            // So, here we reconstruct a full path spec in UTF-8 encoding for later use in string displays.
+            // This utf-8 string will be used to construct the chart database entry if required.
+            wxFileName fnbase(dir_name_base);
+            int nDirs = fnbase.GetDirCount();
+
+            wxFileName file_target(FileList[ifile]);
+            
+            for(int i = 0 ; i < nDirs+1; i++)   // strip off the erroneous intial directories
+                file_target.RemoveDir(0);
+            
+            wxString leftover_path = file_target.GetFullPath();
+            utf8_path = dir_name_base + leftover_path;  // reconstruct a fully utf-8 version
+#endif            
+
+
             //    Validate the file name again, considering MSW's semi-random treatment of case....
             // TODO...something fishy here - may need to normalize saved name?
             if(!file_name.Matches(lowerFileSpec) && !file_name.Matches(filespec) &&
@@ -2101,7 +2178,7 @@ int ChartDatabase::SearchDirAndAddCharts(wxString& dir_name_base,
             }
 
             if( pprog && ( ( ifile % nFileProgressQuantum ) == 0 ) )
-                  pprog->Update( static_cast<int>( ifile * rFileProgressRatio ), full_name );
+                  pprog->Update( static_cast<int>( ifile * rFileProgressRatio ), utf8_path );
 
             ChartTableEntry *pnewChart = NULL;
             bool bAddFinal = true;
@@ -2117,7 +2194,7 @@ int ChartDatabase::SearchDirAndAddCharts(wxString& dir_name_base,
 
             if( collision ) {
                 pEntry = &active_chartTable[collision_ptr->second];
-                table_file_name = wxString::FromUTF8(pEntry->GetpFullPath());
+                table_file_name = pEntry->GetFullSystemPath();
                 file_path_is_same = bthis_dir_in_dB && full_name.IsSameAs(table_file_name);
 
                 // If the chart full file paths are exactly the same, select the newer one.
@@ -2148,7 +2225,7 @@ int ChartDatabase::SearchDirAndAddCharts(wxString& dir_name_base,
                 // Produce the same output without actually calling `CreateChartTableEntry()`.
                 wxLogMessage(wxString::Format(_T("Loading chart data for %s"), msg_fn.c_str()));
             } else {
-                pnewChart = CreateChartTableEntry(full_name, chart_desc);
+                pnewChart = CreateChartTableEntry(full_name, utf8_path, chart_desc);
                 if(!pnewChart)
                 {
                     bAddFinal = false;
@@ -2248,7 +2325,7 @@ bool ChartDatabase::AddChart( wxString &chartfilename, ChartClassDescriptor &cha
     wxString msg_fn(full_name);
     msg_fn.Replace(_T("%"), _T("%%"));
     
-    pnewChart = CreateChartTableEntry(full_name, chart_desc);
+    pnewChart = CreateChartTableEntry(full_name, full_name, chart_desc);
     if(!pnewChart)
     {
         bAddFinal = false;
@@ -2416,7 +2493,7 @@ bool ChartDatabase::AddSingleChart( wxString &ChartFullPath, bool b_force_full_s
  
     //  Get a new magic number
     wxString new_magic;
-    DetectDirChange(dir_name, _T(""), new_magic, 0);
+    DetectDirChange(dir_name, _T(""), _T(""), new_magic, 0);
         
     
     //    Update (clone) the CDI array
@@ -2472,7 +2549,7 @@ bool ChartDatabase::RemoveSingleChart( wxString &ChartFullPath )
  
     //  Walk the chart table, looking for the target
     for(unsigned int i=0 ; i<active_chartTable.GetCount() ; i++) {
-        if( !strcmp( ChartFullPath.mb_str(), GetChartTableEntry(i).GetpFullPath() ) ){
+        if( ChartFullPath.IsSameAs(GetChartTableEntry(i).GetFullSystemPath() ) ){
             active_chartTable.RemoveAt(i);
             break;
         }
@@ -2537,7 +2614,7 @@ ChartBase *ChartDatabase::GetChart(const wxChar *theFilePath, ChartClassDescript
 // Create Chart Table entry by reading chart header info, etc.
 ///////////////////////////////////////////////////////////////////////
 
-ChartTableEntry *ChartDatabase::CreateChartTableEntry(const wxString &filePath, ChartClassDescriptor &chart_desc)
+ChartTableEntry *ChartDatabase::CreateChartTableEntry(const wxString &filePath, wxString &utf8Path, ChartClassDescriptor &chart_desc)
 {
       wxString msg_fn(filePath);
       msg_fn.Replace(_T("%"), _T("%%"));
@@ -2557,7 +2634,7 @@ ChartTableEntry *ChartDatabase::CreateChartTableEntry(const wxString &filePath,
       }
 
 
-      ChartTableEntry *ret_val = new ChartTableEntry(*pch);
+      ChartTableEntry *ret_val = new ChartTableEntry(*pch, utf8Path);
       ret_val->SetValid(true);
 
       delete pch;
@@ -2623,7 +2700,7 @@ wxString ChartDatabase::GetDBChartFileName(int dbIndex)
 {
     if((bValid) && (dbIndex >= 0) && (dbIndex < (int)active_chartTable.size()))
       {
-          return wxString(active_chartTable[dbIndex].GetpFullPath(), wxConvUTF8);
+          return wxString(active_chartTable[dbIndex].GetFullSystemPath());
       }
       else
             return _T("");
@@ -2842,9 +2919,9 @@ void ChartDatabase::ApplyGroupArray(ChartGroupArray *pGroupArray)
             for(unsigned int igroup = 0; igroup < pGroupArray->GetCount(); igroup++)
             {
                   ChartGroup *pGroup = pGroupArray->Item(igroup);
-                  for(unsigned int j=0; j < pGroup->m_element_array.GetCount(); j++)
+                  for(auto& elem : pGroup->m_element_array )
                   {
-                        wxString element_root = pGroup->m_element_array[j]->m_element_name;
+                        wxString element_root = elem->m_element_name;
                         
                         //  The element may be a full single chart name
                         //  If so, add it
@@ -2855,9 +2932,9 @@ void ChartDatabase::ApplyGroupArray(ChartGroupArray *pGroupArray)
                         if(chart_full_path->StartsWith(element_root))
                         {
                               bool b_add = true;
-                              for(unsigned int k=0 ; k < pGroup->m_element_array[j]->m_missing_name_array.GetCount(); k++)
+                              for(unsigned int k=0 ; k < elem->m_missing_name_array.size(); k++)
                               {
-                                    wxString missing_item = pGroup->m_element_array[j]->m_missing_name_array[k];
+                                    wxString missing_item = elem->m_missing_name_array[k];
                                     if(chart_full_path->StartsWith(missing_item))
                                     {
                                           if(chart_full_path->IsSameAs( missing_item )) // missing item is full chart name
