@@ -35,9 +35,9 @@
 #include "wx/tokenzr.h"
 #include <wx/mstream.h>
 
-#ifdef USE_S57
-#include "mygdal/ogr_geometry.h"
-#endif
+#include "config.h"
+
+#include "gdal/ogr_geometry.h"
 
 #include "cutil.h"
 
@@ -545,9 +545,6 @@ int PolyTessGeo::BuildTessGLU()
       }
 
       //  Apply LOD reduction
-    int beforeLOD = ptValid;
-    int afterLOD = beforeLOD;
-   
  
     if(ptValid > 20 && (m_LOD_meters > .01)) {
         std::vector<bool> bool_keep(ptValid, false);
@@ -576,9 +573,6 @@ int PolyTessGeo::BuildTessGLU()
             }
         }
 
-        beforeLOD = ptValid;
-        afterLOD = kept_LOD;
-
         // Copy the lod points back into the vertex buffer
         memcpy(geoPt, LOD_result, kept_LOD * 3 * sizeof(double));
         
@@ -829,7 +823,6 @@ int PolyTessGeo::BuildTess(void)
 //    int nvflags = 0;
 #else
     int allocated = 0;
-    double t0 = 0, t1 = 0;
 #endif
 
 #ifdef USE_POOL
@@ -1583,20 +1576,6 @@ void vertexCallback(GLvoid *vertex, void *polyData)
 
 }
 
-void  vertexCallbackND(GLvoid *vertex)
-{
-    GLdouble *pointer;
-
-    pointer = (GLdouble *) vertex;
-
-
-    double x = pointer[0];
-    double y = pointer[1];
-    
-    int yyp = 4;
-    
-}
-
 /*  combineCallback is used to create a new vertex when edges
  *  intersect.
  */
@@ -1636,6 +1615,7 @@ PolyTriGroup::PolyTriGroup()
     single_buffer = NULL;
     single_buffer_size = 0;
     data_type = DATA_TYPE_DOUBLE;
+    sfactor = 1.0;
     
 
 }
